// (C) 2017-2020 by folkert van heusden, released under AGPL v3.0
#include <string.h>
#include "view_pip.h"
#include "http_server.h"
#include "utils.h"
#include "picio.h"
#include "log.h"

view_pip::view_pip(configuration_t *const cfg, const std::string & id, const std::string & descr, const int width, const int height, resize *const r, const std::vector<view_src_t> & sources, std::vector<target *> *const targets, std::vector<filter *> *const filters, int perc) : view_ss(cfg, id, descr, width, height, false, r, sources, 0.0, targets, filters), perc(perc)
{
}

view_pip::~view_pip()
{
	free(prev_frame);
}

std::string view_pip::get_html(const std::map<std::string, std::string> & pars) const
{
	std::string dim_img;

	if (width != -1)
		dim_img += myformat(" width=%d", width);
	if (height != -1)
		dim_img += myformat(" height=%d", height);

	return myformat("<html><head><style></style></head><body><img src=\"stream.mjpeg?int=%s&view-proxy\"%s></body></html>", url_escape(id).c_str(), url_escape(dim_img).c_str());
}

bool view_pip::get_frame(const encoding_t pe, const int jpeg_quality, uint64_t *ts, int *o_width, int *o_height, uint8_t **frame, size_t *frame_len, const bool use_previous_when_fail)
{
	std::vector<std::thread *> threads;

	instance_t *main_inst = nullptr;
	source *main_source = nullptr;
	find_by_id(cfg, sources.at(0).id, &main_inst, (interface **)&main_source);

	if (!main_source)
		error_exit(false, "Source %s not found", sources.at(0).id.c_str());

	if (main_source->get_class_type() != CT_SOURCE)
		error_exit(false, "Object with id \"%s\" is not of type source", sources.at(0).id.c_str());

	int main_width = 0, main_height = 0;
	uint8_t *main_frame = nullptr;
	size_t main_frame_len = 0;
	bool rc = main_source -> get_frame(E_RGB, 0, ts, &main_width, &main_height, &main_frame, &main_frame_len, true);
	if (!rc)
		return false;

	for(size_t i=1; i<sources.size(); i++) {
		instance_t *pip_inst = nullptr;
		source *pip_source = nullptr;
		find_by_id(cfg, sources.at(i).id, &pip_inst, (interface **)&pip_source);

		if (!pip_source)
			error_exit(false, "Source %s not found", sources.at(i).id.c_str());

		if (pip_source->get_class_type() != CT_SOURCE)
			error_exit(false, "Object with id \"%s\" is not of type source", sources.at(i).id.c_str());

		int pip_width = 0, pip_height = 0;
		uint8_t *pip_frame = nullptr;
		size_t pip_frame_len = 0;

		if (!pip_source -> get_frame(E_RGB, 0, ts, &pip_width, &pip_height, &pip_frame, &pip_frame_len, true)) {
			log(LL_WARNING, "Picture-in-picture frame %s not available", sources.at(i).id.c_str());
			continue;
		}

		picture_in_picture(r, main_frame, main_width, main_height, pip_frame, pip_width, pip_height, perc, sources.at(i).pos);
		free(pip_frame);
	}

	*o_width = main_width;
	*o_height = main_height;

	const size_t frame_out_len = main_width * main_height * 3;
	// FIXME locking
	if (!prev_frame)
		prev_frame = (uint8_t *)allocate_aligned(frame_out_len);

	if (filters) {
		instance_t *inst = find_instance_by_interface(cfg, this);
		apply_filters(inst, this, filters, prev_frame, main_frame, *ts, *o_width, *o_height);
		memcpy(prev_frame, main_frame, frame_out_len);
	}

	if (pe == E_RGB) {
		*frame = main_frame;
		*frame_len = frame_out_len;
		return true;
	}
	else if (pe == E_JPEG) {
		rc = write_JPEG_memory(get_meta(), *o_width, *o_height, jpeg_quality, main_frame, (char **)frame, frame_len);
		free(main_frame);

		return rc;
	}

	log(LL_ERR, "fail return");

	return false;
}
